package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.winhc.repal.entity.*;
import com.winhc.repal.enums.*;
import com.winhc.repal.model.vo.RemindTypeUnReadCountVO;
import com.winhc.repal.model.vo.RepalBillRemindUnReadVO;
import com.winhc.repal.repository.RepalRemindHistoryMapper;
import com.winhc.repal.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.repal.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @description RepalRemindHistory 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalRemindHistoryServiceImpl extends ServiceImpl<RepalRemindHistoryMapper, RepalRemindHistory> implements RepalRemindHistoryService {
    private static final String MESSAGE_PRIFIX = "winhc:repal:key_";
    @Autowired
    private RepalRemindDefinitionService repalRemindDefinitionService;
    @Autowired
    private MessageNoticeService messageNoticeService;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private ThirdPartyAccountService thirdPartyAccountService;
    @Autowired
    private RepalRemindReadInfoService repalRemindReadInfoService;
    @Autowired
    private RepalRemindHistoryMapper repalRemindHistoryMapper;

    @Override
    public RepalBillRemindUnReadVO getRemindUnReadVO(List<RepalBill> repalBills,Long userId) {
        RepalBillRemindUnReadVO result = new RepalBillRemindUnReadVO(0,0,0,0,0);
        for(RepalBill repalBill:repalBills){
            RepalRemindReadInfo repalRemindReadInfo = repalRemindReadInfoService.getReadInfoByUserId(userId,repalBill.getId(),repalBill.getRepalBookId());
            RepalBillRemindUnReadVO readVO = repalRemindHistoryMapper.getRemindUnReadVO(repalRemindReadInfo.getRiskRemindId(),
                    repalRemindReadInfo.getRankRemindId(),repalRemindReadInfo.getOverdueRemindId(),repalRemindReadInfo.getFinanceRemindId(),repalBill.getRepalBookId(),repalBill.getId());
            result.setUnreadRankCount(result.getUnreadRankCount()+readVO.getUnreadRankCount());
            result.setUnreadRiskCount(result.getUnreadRiskCount()+readVO.getUnreadRiskCount());
            result.setUnreadOverdueCount(result.getUnreadOverdueCount()+readVO.getUnreadOverdueCount());
            result.setUnreadFinanceCount(result.getUnreadFinanceCount()+readVO.getUnreadFinanceCount());
        }
        result.setUnreadTotalCount(result.getUnreadFinanceCount()+result.getUnreadRankCount()+result.getUnreadRiskCount()+result.getUnreadOverdueCount());
        return result;
    }

    @Override
    public boolean insertRankRemind(RepalBill repalBill, RepalBillDiagnosis diagnosis) {
        if (repalBill.getRepalBillStage() == null) {
            return true;
        }
        LocalDate endDate = repalBill.getEndDate();
        if (endDate == null) {
            endDate = LocalDate.now().minusDays(1);
        }
        OverdueEnum overdueEnum = getDaysBetween(endDate);
        LambdaQueryWrapper<RepalRemindDefinition> qw = Wrappers.lambdaQuery(RepalRemindDefinition.class);
        qw.eq(RepalRemindDefinition::getRemindType, RemindEnum.账款评级.name());
        if (RepalBillStatusEnum.PROGRESSING.getCode().equals(repalBill.getRepalBillStatus())
                || !RepalBillStageEnum.NON_PROSECUTE.getCode().equals(repalBill.getRepalBillStage())) {
            qw.eq(RepalRemindDefinition::getOpportunity, diagnosis.getDiagnosisGrade());
        } else {
            qw.eq(RepalRemindDefinition::getOpportunity, diagnosis.getDiagnosisGrade() + "+" + overdueEnum.getCode());
        }
        qw.eq(RepalRemindDefinition::getRepalBillStatus, repalBill.getRepalBillStatus());
        qw.eq(RepalRemindDefinition::getRepalBillStage, repalBill.getRepalBillStage());
        RepalRemindDefinition accountRemindDefinition = repalRemindDefinitionService.getOne(qw, false);
        if (accountRemindDefinition != null) {
            String changeStr = "下降";
            String changeStr1 = "下降";
            if (repalBill.getDiagnosisScore().compareTo(repalBill.getPreDiagnosisScore() == null ? new BigDecimal(0)
                    : repalBill.getPreDiagnosisScore()) > 0) {
                changeStr = "上升";
                changeStr1 = "提高";
            }
            RepalRemindHistory accountRemindHistory = new RepalRemindHistory();
            accountRemindHistory.setUserId(repalBill.getUserId())
                    .setRepalBookId(repalBill.getRepalBookId())
                    .setRepalBillId(repalBill.getId())
                    .setTrendId(diagnosis.getId().toString())
                    .setTrendContent(repalBill.getPreDiagnosisId() != null ? repalBill.getPreDiagnosisId().toString() : null)
                    .setRemindTime(LocalDateTime.now())
                    .setCompanyName(repalBill.getCustomerName())
                    .setRemindTitle(diagnosis.getDiagnosisGrade() + "（" + diagnosis.getDiagnosisScore().stripTrailingZeros().toPlainString() + "分）")
                    .setRemindContent(StrUtil.format(accountRemindDefinition.getRemindContent(),
                            changeStr,
                            changeStr1,
                            overdueEnum.name()))
                    .setFunctionList(accountRemindDefinition.getFunctionList())
                    .setRemindType(RemindEnum.账款评级.getCode());
            save(accountRemindHistory);
            //推送
            //查出组织下所有已经授权的成员
            List<ThirdPartyAccount> accounts = thirdPartyAccountService.list(Wrappers.lambdaQuery(ThirdPartyAccount.class)
                    .eq(ThirdPartyAccount::getGroupId,repalBill.getRepalGroupId())
                    .eq(ThirdPartyAccount::getDeleted,Dict.DeletedStatusEnunm.否.getCode())
                    .eq(ThirdPartyAccount::getAppId,0)
                    .isNotNull(ThirdPartyAccount::getMobileNo));
            List<String> mobiles = accounts.stream().map(ThirdPartyAccount::getMobileNo).collect(Collectors.toList());
            String redisKey = MESSAGE_PRIFIX + repalBill.getRepalGroupId() + "_" + RemindEnum.账款评级.getCode();
            if (CollUtil.isNotEmpty(mobiles) && redisUtil.get(redisKey) == null && repalBill.getPreDiagnosisId() != null) {
                for(String mobileNo:mobiles) {
                    Map<String, String> paraMap = new HashMap<>();
                    paraMap.put("company", repalBill.getCustomerName());
                    paraMap.put("pregrade", repalBill.getPreDiagnosisGrade());
                    paraMap.put("prescore", repalBill.getPreDiagnosisScore().toPlainString());
                    paraMap.put("change", changeStr);
                    paraMap.put("grade", repalBill.getDiagnosisGrade());
                    paraMap.put("score", repalBill.getDiagnosisScore().toPlainString());
                    paraMap.put("change1", changeStr1);
                    messageNoticeService.sendSms(mobileNo, MsgNameEnum.评级变动短信提醒, paraMap);
                }
                redisUtil.setEx(redisKey, "done", calTTL(), TimeUnit.MILLISECONDS);
            }
            //掐头去尾压进accountbill.remindInfo中
            String remindContent = accountRemindHistory.getRemindContent();
            if (remindContent.length() > 100) {
                remindContent = remindContent.substring(0, 100);
            }
            accountRemindHistory.setFunctionList(null).setRemindContent(remindContent);
            repalBill.setRemindInfo(JSON.toJSONString(accountRemindHistory));
        }
        return true;
    }

    private OverdueEnum getDaysBetween(LocalDate endDate) {
        long delta = LocalDate.now().toEpochDay() - endDate.toEpochDay();
        OverdueEnum overdueEnum = OverdueEnum.其他;
        if (delta <= 0 && delta >= -7) {
            overdueEnum = OverdueEnum.即将逾期;
        } else if (delta > 0 && delta < 90) {
            overdueEnum = OverdueEnum.已逾期;
        } else if (delta >= 90 && delta < 180) {
            overdueEnum = OverdueEnum.已逾期三个月;
        } else if (delta >= 180 && delta < 365) {
            overdueEnum = OverdueEnum.已逾期半年;
        } else if (delta >= 365 && delta < 365 * 3) {
            overdueEnum = OverdueEnum.已逾期一年;
        } else if (delta >= 365 * 3) {
            overdueEnum = OverdueEnum.已逾期三年;
        }
        return overdueEnum;
    }

    /**
     * 计算当天剩余的毫秒时间
     * @return
     */
    private Long calTTL(){
        LocalDateTime midnight = LocalDateTime.now().plusDays(1).withHour(0).withMinute(0).withSecond(0).withNano(0);
        return ChronoUnit.MILLIS.between(LocalDateTime.now(),midnight);
    }
}
